@@ -4,6 +4,8 @@ use core::ops::Deref;
 
 use cortex_m::interrupt::free;
 
+use num_traits::Float; // Float rounding.
+
 use crate::{
     clocks::Clocks,
     pac::{self, RCC},
@@ -173,8 +175,7 @@ impl Default for DfsdmConfig {
             filter_order: FilterOrder::Sinc4, // From the PDM mic AN
             filter_oversampling_ratio: 64,    // From the PDM mic AN
             integrator_oversampling_ratio: 1, // From the PDM mic AN
-            right_shift_bits: 0x02,           // From the PDM mic AN
-            // right_shift_bits: 0x00, // From the PDM mic AN
+            right_shift_bits: 0,              // PDM mic AN uses 0x02
             spi_clock: SpiClock::Internal,
         }
     }
@@ -214,7 +215,7 @@ where
         // Divider = DFSDM Clock Source / (AUDIO_SAMPLING_FREQUENCY ×
         // DECIMATION_FACTOR)
         // (Note that for a 3.072Mhz clock , 64 decimation factor and 48K audio, te divider
-        // works out to 1)
+        // works out to 1. With a 12.288Mhz clock, and the same factors, it's 4.)
 
         let clock_speed = match config.clock_src {
             DfsdmClockSrc::SysClk => clock_cfg.sysclk(),
@@ -222,8 +223,12 @@ where
             DfsdmClockSrc::AudioClk => clock_cfg.sai1_speed(),
         };
 
+        // We want it to round to nearest divider, not floor round. Let's say our clock speed is slightly lower than
+        // the target - we might get a divider calculated to be 3.99. With integer math, that would round down to 3,
+        // and be effectively wrong - hence the floats.
         let divider =
-            clock_speed / (config.sampling_freq * config.filter_oversampling_ratio as u32);
+            clock_speed as f32 / (config.sampling_freq * config.filter_oversampling_ratio as u32) as f32;
+        let divider = divider.round() as u8;
 
         // 1- 255: Defines the division of system clock for the serial clock output for CKOUT signal in range 2 -
         // 256 (Divider = CKOUTDIV+1).
@@ -257,8 +262,6 @@ where
         // channel analog watchdog filter parameters and channel short-circuit detector
         // parameters. Configurations are defined in CHyAWSCDR register.
 
-        // todo: Software trigger
-
         Self { regs, config }
     }
 
@@ -269,9 +272,6 @@ where
     /// enable bit CHEN in CHyCFGR1 and FLTx enable bit DFEN in
     /// FLTxCR1).
     pub fn enable(&mut self) {
-        // todo temp!
-        // self.regs.ch0.cfgr1.modify(|_, w| unsafe { w.datmpx().bits(2) });
-
         self.regs.ch0.cfgr1.modify(|_, w| w.dfsdmen().set_bit());
     }
 
@@ -287,6 +287,8 @@ where
     /// Digital filter x FLTx (x=0..3) is enabled by setting DFEN=1 in the
     /// FLTxCR1 register. Once FLTx is enabled (DFEN=1), both Sincx
     /// digital filter unit and integrator unit are reinitialized.
+    /// Note that this function sets `DFEN`, so run it `after` configuring other settings such
+    /// as DMA.
     pub fn enable_filter(&mut self, filter: Filter, channel: DfsdmChannel) {
         // Setting RCONT in the FLTxCR1 register causes regular conversions to execute in
         // continuous mode. RCONT=1 means that the channel selected by RCH[2:0] is converted
@@ -403,86 +405,88 @@ where
         // Once the channel is enabled, it receives serial data from the external Σ∆ modulator or
         // parallel internal data sources (ADCs or CPU/DMA wire from memory).
 
+        // Note that we must set some settings like `dtrbs` before enabling the channel.
+
         match channel {
             DfsdmChannel::C0 => unsafe {
-                self.regs.ch0.cfgr1.modify(|_, w| {
-                    w.spicksel().bits(self.config.spi_clock as u8);
-                    w.chen().set_bit()
-                });
                 self.regs
                     .ch0
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
-            },
-            DfsdmChannel::C1 => unsafe {
-                self.regs.ch1.cfgr1.modify(|_, w| {
+                self.regs.ch0.cfgr1.modify(|_, w| {
                     w.spicksel().bits(self.config.spi_clock as u8);
                     w.chen().set_bit()
                 });
+            },
+            DfsdmChannel::C1 => unsafe {
                 self.regs
                     .ch1
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
-            },
-            DfsdmChannel::C2 => unsafe {
-                self.regs.ch2.cfgr1.modify(|_, w| {
+                self.regs.ch1.cfgr1.modify(|_, w| {
                     w.spicksel().bits(self.config.spi_clock as u8);
                     w.chen().set_bit()
                 });
+            },
+            DfsdmChannel::C2 => unsafe {
                 self.regs
                     .ch2
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
-            },
-            DfsdmChannel::C3 => unsafe {
-                self.regs.ch3.cfgr1.modify(|_, w| {
+                self.regs.ch2.cfgr1.modify(|_, w| {
                     w.spicksel().bits(self.config.spi_clock as u8);
                     w.chen().set_bit()
                 });
+            },
+            DfsdmChannel::C3 => unsafe {
                 self.regs
                     .ch3
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
-            },
-            DfsdmChannel::C4 => unsafe {
-                self.regs.ch4.cfgr1.modify(|_, w| {
+                self.regs.ch3.cfgr1.modify(|_, w| {
                     w.spicksel().bits(self.config.spi_clock as u8);
                     w.chen().set_bit()
                 });
+            },
+            DfsdmChannel::C4 => unsafe {
                 self.regs
                     .ch4
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
-            },
-            DfsdmChannel::C5 => unsafe {
-                self.regs.ch5.cfgr1.modify(|_, w| {
+                self.regs.ch4.cfgr1.modify(|_, w| {
                     w.spicksel().bits(self.config.spi_clock as u8);
                     w.chen().set_bit()
                 });
+            },
+            DfsdmChannel::C5 => unsafe {
                 self.regs
                     .ch5
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
-            },
-            DfsdmChannel::C6 => unsafe {
-                self.regs.ch6.cfgr1.modify(|_, w| {
+                self.regs.ch5.cfgr1.modify(|_, w| {
                     w.spicksel().bits(self.config.spi_clock as u8);
                     w.chen().set_bit()
                 });
+            },
+            DfsdmChannel::C6 => unsafe {
                 self.regs
                     .ch6
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
-            },
-            DfsdmChannel::C7 => unsafe {
-                self.regs.ch7.cfgr1.modify(|_, w| {
+                self.regs.ch6.cfgr1.modify(|_, w| {
                     w.spicksel().bits(self.config.spi_clock as u8);
                     w.chen().set_bit()
                 });
+            },
+            DfsdmChannel::C7 => unsafe {
                 self.regs
                     .ch7
                     .cfgr2
                     .modify(|_, w| w.dtrbs().bits(self.config.right_shift_bits));
+                self.regs.ch7.cfgr1.modify(|_, w| {
+                    w.spicksel().bits(self.config.spi_clock as u8);
+                    w.chen().set_bit()
+                });
             },
         }
     }
@@ -714,13 +718,12 @@ where
             #[cfg(not(any(feature = "l4")))]
             Filter::F3 => &self.regs.flt3.rdatar as *const _ as u32,
         };
-        // let periph_addr = ch1.datinr as *const _ as u32; // todo try it
 
         // todo: Injected support. Should just need to add the option flag and enable `jdmaen()` bits
         // todo instead of `rdmaen()`, and use the `jdatar` periph addr.
 
         // todo: Do we want this? If so, where?
-        // self.start_conversion(filter);
+        self.start_conversion(filter);
 
         #[cfg(feature = "h7")]
         let len = len as u32;
