@@ -371,6 +371,7 @@ pub mod dac;
     feature = "l4x2",
     feature = "l412",
     feature = "l4x3",
+    feature = "l4x6",
     feature = "g0",
     feature = "g4",
     feature = "wb",
@@ -427,6 +428,17 @@ feature = "wl",
 )))]
 pub mod qspi;
 
+// Note: Some F4 variants support RNG, but we haven't figured out the details yet. Send a PR if interested.
+#[cfg(not(any(
+    feature = "f3",
+    feature = "f4",
+    feature = "g030",
+    feature = "g031",
+    feature = "g070",
+    feature = "g071"
+)))]
+pub mod rng;
+
 pub mod rtc;
 
 #[cfg(not(any(
